#!/bin/sh

# NAME:
#	rte.sh - compose a GPX route from a list of waypoints
#
# SYNOPSIS:
#	rte.sh [options]
#
# DESCRIPTION:
#	This script uses the waypoint data saved by rtept.sh
#	to compose new routes in valid GPX format.
#
#	Options:
#
#	-e "ext"
#		When looking for rtept data use "ext" ('.rtept')
#
#	-g	output GPX header
#
#	-i "input"
#		read list of waypoints and tags from "input" (stdin).
#
#	-n "name"
#		use "name" for the route name.
#		Only needed if not using 'rte' tags in "input".
#
#	-w "waypoints"
#		identifies where to look for rtept data
#
#	Valid input conists of a sequence of 'tag' 'name' records.
#	If 'tag' is skipped it is assumed to be 'rtept'.
#
#	Other tags include:
#
#	'gpx'	output a GPX header (ignored if it appears more than
#		once)
#
#	'rte'	start a new route (close any open one)
#	
# SEE ALSO:
#	GPX.py(1), rtept.sh(1)
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#


# RCSid:
#	$Id: rte.sh,v 1.2 2021/08/28 20:08:33 sjg Exp $
#
#	@(#) Copyright (c) 2021 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

# extract waypoints from a GPX file,
# save each in its own file.

error() {
    echo "$ERROR: $@" >&2
    exit 1
}

Exists() {
    case "$1" in
    -?) _t=$1; shift;;
    *) _t=-s;;
    esac
    for f in "$@"
    do
        test $_t $f || continue
        echo $f
        return 0
    done
    return 1
}

opt_e=.rtept
opt_str='e:gi:n:w:'
. setopts.sh

gpx_once=:
gpx_opened=
gpx_open() {
    $gpx_once 0
    gpx_once=return
    gpx_opened=1
    cat <<EOH
<?xml version="1.0"?>
<gpx version="1.1" creator="OpenCPN" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns="http://www.topografix.com/GPX/1/1" xmlns:gpxx="http://www.garmin.com/xmlschemas/GpxExtensions/v3" xsi:schemaLocation="http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd" xmlns:opencpn="http://www.opencpn.org">
EOH
}

rte_opened=
rte_open() {
    [ -z "$rte_opened" ] || echo "  </rte>"
    echo "  <rte><name>$1</name>"
    rte_opened="$1"
}

[ -z "$opt_g" ] || gpx_open
[ -z "$opt_n" ] || rte_open "$opt_n"
cat $opt_i | (
while read tag name
do
    case "$tag" in
    gpx) gpx_open;;
    rte) rte_open "$name";;
    *)
        name=${name:-$tag}
        rtept=`Exists $name $opt_w/$name $opt_w/$name$opt_e`
        if [ -z "$rtept" ]; then
            echo WARNING: cannot find $name
            continue
	fi
        cat $rtept
        ;;
    esac
done
[ -z "$rte_opened" ] || echo "  </rte>"
[ -z "$gpx_opened" ] || echo "</gpx>"

)
